@@ -8,7 +8,7 @@
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful, 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
@@ -25,11 +25,7 @@
 #-------------------------------------------------------------------------
 from gramps.gen.const import GRAMPS_LOCALE as glocale
 _ = glocale.translation.gettext
-import sys
-if sys.version_info[0] < 3:
-    import cPickle as pickle
-else:
-    import pickle
+import pickle
 
 import logging
 _LOG = logging.getLogger(".objectentries")
@@ -48,7 +44,7 @@ from gi.repository import Pango
 # Gramps modules
 #
 #-------------------------------------------------------------------------
-from gramps.gen.lib import (Place, Source, MediaObject, Note)
+from gramps.gen.lib import (Place, Source, Media, Note)
 from .editplace import EditPlace
 from .editsource import EditSource
 from .editmedia import EditMedia
@@ -63,7 +59,7 @@ from gramps.gen.display.place import displayer as place_displayer
 # ObjEntry
 #
 #-------------------------------------------------------------------------
-class ObjEntry(object):
+class ObjEntry:
     """
     Handles the selection of a existing or new Object. Supports Drag and Drop
     to select the object.
@@ -75,20 +71,24 @@ class ObjEntry(object):
     SHARE_STR = ""
     ADD_STR = ""
     DEL_STR = ""
+    _DND_TYPE = None
+    _DND_ICON = None
 
-    def __init__(self, dbstate, uistate, track, label, set_val, 
+    def __init__(self, dbstate, uistate, track, label, label_event_box, set_val,
                  get_val, add_edt, share, callback=None):
         """Pass the dbstate and uistate and present track.
             label is a Gtk.Label that shows the persent value
+            label_event_box is a Gtk.EventBox used for allowing drag from the label
             set_val is function that is called when handle changes, use it
                 to update the calling module
             get_val is function that is called to obtain handle from calling
                 module
             share is the Gtk.Button to call the object selector or del connect
-            add_edt is the Gtk.Button with add or edit value. Pass None if 
+            add_edt is the Gtk.Button with add or edit value. Pass None if
                 this button should not be present.
         """
         self.label = label
+        self.label_event_box = label_event_box
         self.add_edt = add_edt
         self.share = share
         self.dbstate = dbstate
@@ -98,9 +98,7 @@ class ObjEntry(object):
         self.uistate = uistate
         self.track = track
         self.callback = callback
-        
-        #connect drag and drop
-        self._init_dnd()
+
         #set the object specific code
         self._init_object()
 
@@ -131,7 +129,7 @@ class ObjEntry(object):
         if self.add_edt is not None:
             self.add_edt.connect('clicked', self.add_edt_clicked)
         self.share.connect('clicked', self.share_clicked)
-        
+
         if not self.db.readonly and not name:
             if self.add_edt is None:
                 self.label.set_text(self.EMPTY_TEXT_RED)
@@ -144,10 +142,33 @@ class ObjEntry(object):
         if self.callback:
             self.callback()
 
-    def _init_dnd(self):
-        """inheriting objects must set this
+    def _update_dnd_capability(self):
         """
-        pass
+        Connect drag and drop of object entry
+            Drop is always allowed
+            Drag only allowed when object is set
+        """
+        if not self._DND_TYPE:
+            return
+        if not self.label.drag_dest_get_target_list():
+            self.label.drag_dest_set(Gtk.DestDefaults.ALL,
+                                     [self._DND_TYPE.target()],
+                                     Gdk.DragAction.COPY)
+            self.label.connect('drag_data_received', self.drag_data_received)
+        # Set the drag action from this box
+        if self.get_val():
+            if not self.label.drag_source_get_target_list():
+                self.label_event_box.drag_source_set(
+                    Gdk.ModifierType.BUTTON1_MASK,
+                    [self._DND_TYPE.target()], Gdk.DragAction.COPY)
+                if self._DND_ICON:
+                    self.label_event_box.drag_source_set_icon_name(
+                        self._DND_ICON)
+                self.label_event_box.connect('drag_data_get',
+                                             self.drag_data_get)
+        else:
+            if self.label.drag_source_get_target_list():
+                self.label_event_box.drag_source_unset()
 
     def _init_object(self):
         """inheriting objects can use this to set extra variables
@@ -193,10 +214,15 @@ class ObjEntry(object):
 
     def drag_data_received(self, widget, context, x, y, selection, info, time):
         (drag_type, idval, obj, val) = pickle.loads(selection.get_data())
-        
         data = self.get_from_handle(obj)
         self.obj_added(data)
-        
+
+    def drag_data_get(self, widget, context, sel_data, info, time):
+        # get the selected object, returning if not is defined
+        if info == self._DND_TYPE.app_id:
+            data = (self._DND_TYPE.drag_type, id(self), self.get_val(), 0)
+            sel_data.set(self._DND_TYPE.atom_drag_type, 8, pickle.dumps(data))
+
     def obj_added(self, data):
         """ callback from adding an object to the entry"""
         self.set_val(data.handle)
@@ -233,28 +259,29 @@ class ObjEntry(object):
 
         if use_add:
             image = Gtk.Image()
-            image.set_from_stock(Gtk.STOCK_REMOVE, Gtk.IconSize.BUTTON)
+            image.set_from_icon_name('list-remove', Gtk.IconSize.BUTTON)
             image.show()
             self.share.add(image)
             self.share.set_tooltip_text(self.DEL_STR)
             if self.add_edt is not None:
                 image = Gtk.Image()
-                image.set_from_stock(Gtk.STOCK_EDIT, Gtk.IconSize.BUTTON)
+                image.set_from_icon_name('gtk-edit', Gtk.IconSize.BUTTON)
                 image.show()
                 self.add_edt.add(image)
                 self.add_edt.set_tooltip_text(self.EDIT_STR)
         else:
             image = Gtk.Image()
-            image.set_from_stock(Gtk.STOCK_INDEX, Gtk.IconSize.BUTTON)
+            image.set_from_icon_name('gtk-index', Gtk.IconSize.BUTTON)
             image.show()
             self.share.add(image)
             self.share.set_tooltip_text(self.SHARE_STR)
             if self.add_edt is not None:
                 image = Gtk.Image()
-                image.set_from_stock(Gtk.STOCK_ADD, Gtk.IconSize.BUTTON)
+                image.set_from_icon_name('list-add', Gtk.IconSize.BUTTON)
                 image.show()
                 self.add_edt.add(image)
                 self.add_edt.set_tooltip_text(self.ADD_STR)
+        self._update_dnd_capability()
 
 class PlaceEntry(ObjEntry):
     """
@@ -268,30 +295,15 @@ class PlaceEntry(ObjEntry):
     SHARE_STR = _('Select an existing place')
     ADD_STR = _('Add a new place')
     DEL_STR = _('Remove place')
-    
-    def __init__(self, dbstate, uistate, track, label, set_val, 
+    _DND_TYPE = DdTargets.PLACE_LINK
+    _DND_ICON = 'gramps-place'
+
+    def __init__(self, dbstate, uistate, track, label, label_event_box, set_val,
                  get_val, add_edt, share, skip=[]):
-        ObjEntry.__init__(self, dbstate, uistate, track, label, set_val, 
+        ObjEntry.__init__(self, dbstate, uistate, track, label, label_event_box, set_val,
                  get_val, add_edt, share)
         self.skip = skip
 
-    def _init_dnd(self):
-        """connect drag and drop of places
-        """
-        self.label.drag_dest_set(Gtk.DestDefaults.ALL, [], Gdk.DragAction.COPY)
-        tglist = Gtk.TargetList.new([])
-        tglist.add(DdTargets.PLACE_LINK.atom_drag_type, DdTargets.PLACE_LINK.target_flags,
-                   DdTargets.PLACE_LINK.app_id)
-        self.label.drag_dest_set_target_list(tglist)
-        self.label.connect('drag_data_received', self.drag_data_received)
-
-    def drag_data_received(self, widget, context, x, y, selection, info, time):
-        (drag_type, idval, obj, val) = pickle.loads(selection.get_data())
-
-        if obj not in self.skip:
-            data = self.get_from_handle(obj)
-            self.obj_added(data)
-
     def get_from_handle(self, handle):
         """ return the object given the hande
         """
@@ -309,7 +321,7 @@ class PlaceEntry(ObjEntry):
             place = obj
             func = self.after_edit
         try:
-            EditPlace(self.dbstate, self.uistate, self.track, 
+            EditPlace(self.dbstate, self.uistate, self.track,
                       place, func)
         except WindowActiveError:
             pass
@@ -323,28 +335,20 @@ class SourceEntry(ObjEntry):
     Handles the selection of a existing or new Source. Supports Drag and Drop
     to select a source.
     """
-    EMPTY_TEXT = "<i>%s</i>" % _('First add a source using the buttons')
+    EMPTY_TEXT = "<i>%s</i>" % _('To select a source, use drag-and-drop '
+                                 'or use the buttons')
     EMPTY_TEXT_RED = "<i>%s</i>" % _('First add a source using the button')
     EDIT_STR = _('Edit source')
     SHARE_STR = _('Select an existing source')
     ADD_STR = _('Add a new source')
     DEL_STR = _('Remove source')
-    
-    def __init__(self, dbstate, uistate, track, label, set_val, 
-                 get_val, add_edt, share, callback):
-        ObjEntry.__init__(self, dbstate, uistate, track, label, set_val, 
-                 get_val, add_edt, share, callback)
+    _DND_TYPE = DdTargets.SOURCE_LINK
+    _DND_ICON = 'gramps-source'
 
-    def _init_dnd(self):
-        """connect drag and drop of sources
-        """
-        self.label.drag_dest_set(Gtk.DestDefaults.ALL, [], Gdk.DragAction.COPY)
-        tglist = Gtk.TargetList.new([])
-        tglist.add(DdTargets.SOURCE_LINK.atom_drag_type,
-                   DdTargets.SOURCE_LINK.target_flags,
-                   DdTargets.SOURCE_LINK.app_id)
-        self.label.drag_dest_set_target_list(tglist)
-        self.label.connect('drag_data_received', self.drag_data_received)
+    def __init__(self, dbstate, uistate, track, label, label_event_box, set_val,
+                 get_val, add_edt, share, callback):
+        ObjEntry.__init__(self, dbstate, uistate, track, label, label_event_box, set_val,
+                 get_val, add_edt, share, callback)
 
     def get_from_handle(self, handle):
         """ return the object given the handle
@@ -362,7 +366,7 @@ class SourceEntry(ObjEntry):
             source = obj
             func = self.after_edit
         try:
-            EditSource(self.dbstate, self.uistate, self.track, 
+            EditSource(self.dbstate, self.uistate, self.track,
                        source, func)
         except WindowActiveError:
             pass
@@ -384,47 +388,39 @@ class MediaEntry(ObjEntry):
     SHARE_STR = _('Select an existing media object')
     ADD_STR = _('Add a new media object')
     DEL_STR = _('Remove media object')
-    
-    def __init__(self, dbstate, uistate, track, label, set_val, 
-                 get_val, add_edt, share):
-        ObjEntry.__init__(self, dbstate, uistate, track, label, set_val, 
-                 get_val, add_edt, share)
+    _DND_TYPE = DdTargets.MEDIAOBJ
+    _DND_ICON = 'gramps-media'
 
-    def _init_dnd(self):
-        """connect drag and drop of places
-        """
-        self.label.drag_dest_set(Gtk.DestDefaults.ALL, [], Gdk.DragAction.COPY)
-        tglist = Gtk.TargetList.new([])
-        tglist.add(DdTargets.MEDIAOBJ.atom_drag_type, DdTargets.MEDIAOBJ.target_flags,
-                   DdTargets.MEDIAOBJ.app_id)
-        self.label.drag_dest_set_target_list(tglist)
-        self.label.connect('drag_data_received', self.drag_data_received)
+    def __init__(self, dbstate, uistate, track, label, label_event_box, set_val,
+                 get_val, add_edt, share):
+        ObjEntry.__init__(self, dbstate, uistate, track, label, label_event_box, set_val,
+                 get_val, add_edt, share)
 
     def get_from_handle(self, handle):
         """ return the object given the hande
         """
-        return self.db.get_object_from_handle(handle)
+        return self.db.get_media_from_handle(handle)
 
     def get_label(self, object):
         return "%s [%s]" % (object.get_description(), object.gramps_id)
 
     def call_editor(self, obj=None):
         if obj is None:
-            object = MediaObject()
+            object = Media()
             func = self.obj_added
         else:
             object = obj
             func = self.after_edit
         try:
-            EditMedia(self.dbstate, self.uistate, self.track, 
+            EditMedia(self.dbstate, self.uistate, self.track,
                       object, func)
         except WindowActiveError:
             pass
 
     def call_selector(self):
-        cls = SelectorFactory('MediaObject')
+        cls = SelectorFactory('Media')
         return cls(self.dbstate, self.uistate, self.track)
-    
+
 # FIXME isn't used anywhere
 class NoteEntry(ObjEntry):
     """
@@ -438,10 +434,12 @@ class NoteEntry(ObjEntry):
     SHARE_STR = _('Select an existing note')
     ADD_STR = _('Add a new note')
     DEL_STR = _('Remove note')
-    
-    def __init__(self, dbstate, uistate, track, label, set_val, 
+    _DND_TYPE = DdTargets.NOTE_LINK
+    _DND_ICON = 'gramps-notes'
+
+    def __init__(self, dbstate, uistate, track, label, label_event_box, set_val,
                  get_val, add_edt, share):
-        ObjEntry.__init__(self, dbstate, uistate, track, label, set_val, 
+        ObjEntry.__init__(self, dbstate, uistate, track, label, label_event_box, set_val,
                  get_val, add_edt, share)
         self.notetype = None
 
@@ -455,16 +453,6 @@ class NoteEntry(ObjEntry):
         """
         return self.notetype
 
-    def _init_dnd(self):
-        """connect drag and drop of places
-        """
-        self.label.drag_dest_set(Gtk.DestDefaults.ALL, [], Gdk.DragAction.COPY)
-        tglist = Gtk.TargetList.new([])
-        tglist.add(DdTargets.NOTE_LINK.atom_drag_type, DdTargets.NOTE_LINK.target_flags,
-                   DdTargets.NOTE_LINK.app_id)
-        self.label.drag_dest_set_target_list(tglist)
-        self.label.connect('drag_data_received', self.drag_data_received)
-
     def get_from_handle(self, handle):
         """ return the object given the hande
         """
@@ -485,7 +473,7 @@ class NoteEntry(ObjEntry):
             note = obj
             func = self.after_edit
         try:
-            EditNote(self.dbstate, self.uistate, self.track, 
+            EditNote(self.dbstate, self.uistate, self.track,
                          note, func)
         except WindowActiveError:
             pass
