@@ -17,8 +17,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
+from gramps.gen.plug._pluginreg import newplugin, STABLE, EXPORT
+from gramps.gen.const import GRAMPS_LOCALE as glocale
+_ = glocale.translation.gettext
 
-MODULE_VERSION="4.2" 
+MODULE_VERSION="5.1"
 
 #------------------------------------------------------------------------
 #
@@ -27,10 +30,10 @@ MODULE_VERSION="4.2"
 #------------------------------------------------------------------------
 
 plg = newplugin()
-plg.id    = 'ex_csv'
-plg.name  = _("Comma Separated Values Spreadsheet (CSV)")
-plg.name_accell  = _("Comma _Separated Values Spreadsheet (CSV)")
-plg.description =  _("CSV is a common spreadsheet format.")
+plg.id = 'ex_csv'
+plg.name = _("Comma Separated Values Spreadsheet (CSV)")
+plg.name_accell = _("Comma _Separated Values Spreadsheet (CSV)")
+plg.description = _("CSV is a common spreadsheet format.")
 plg.version = '1.0'
 plg.gramps_target_version = MODULE_VERSION
 plg.status = STABLE
@@ -48,10 +51,10 @@ plg.extension = "csv"
 #------------------------------------------------------------------------
 
 plg = newplugin()
-plg.id    = 'ex_webfamtree'
-plg.name  = _('Web Family Tree')
-plg.name_accell  = _('_Web Family Tree')
-plg.description =  _("Web Family Tree format")
+plg.id = 'ex_webfamtree'
+plg.name = _('Web Family Tree')
+plg.name_accell = _('_Web Family Tree')
+plg.description = _("Web Family Tree format")
 plg.version = '1.0'
 plg.gramps_target_version = MODULE_VERSION
 plg.status = STABLE
@@ -69,10 +72,10 @@ plg.extension = "wft"
 #------------------------------------------------------------------------
 
 plg = newplugin()
-plg.id    = 'ex_ged'
-plg.name  = _('GEDCOM')
-plg.name_accell  = _('GE_DCOM')
-plg.description =  _('GEDCOM is used to transfer data between genealogy programs. '
+plg.id = 'ex_ged'
+plg.name = _('GEDCOM')
+plg.name_accell = _('GE_DCOM')
+plg.description = _('GEDCOM is used to transfer data between genealogy programs. '
                 'Most genealogy software will accept a GEDCOM file as input.')
 plg.version = '1.0'
 plg.gramps_target_version = MODULE_VERSION
@@ -91,10 +94,10 @@ plg.extension = "ged"
 #------------------------------------------------------------------------
 
 plg = newplugin()
-plg.id    = 'ex_geneweb'
-plg.name  = _('GeneWeb')
-plg.name_accell  = _('_GeneWeb')
-plg.description =  _('GeneWeb is a web based genealogy program.')
+plg.id = 'ex_geneweb'
+plg.name = _('GeneWeb')
+plg.name_accell = _('_GeneWeb')
+plg.description = _('GeneWeb is a web based genealogy program.')
 plg.version = '1.0'
 plg.gramps_target_version = MODULE_VERSION
 plg.status = STABLE
@@ -107,15 +110,15 @@ plg.extension = "gw"
 
 #------------------------------------------------------------------------
 #
-# GRAMPS package (portable XML)
+# Gramps package (portable XML)
 #
 #------------------------------------------------------------------------
 
 plg = newplugin()
-plg.id    = 'ex_gpkg'
-plg.name  = _('Gramps XML Package (family tree and media)')
-plg.name_accell  = _('Gra_mps XML Package (family tree and media)')
-plg.description =  _('Gramps package is an archived XML family tree together '
+plg.id = 'ex_gpkg'
+plg.name = _('Gramps XML Package (family tree and media)')
+plg.name_accell = _('Gra_mps XML Package (family tree and media)')
+plg.description = _('Gramps package is an archived XML family tree together '
                  'with the media object files.')
 plg.version = '1.0'
 plg.gramps_target_version = MODULE_VERSION
@@ -129,15 +132,15 @@ plg.extension = "gpkg"
 
 #------------------------------------------------------------------------
 #
-# GRAMPS XML database
+# Gramps XML database
 #
 #------------------------------------------------------------------------
 
 plg = newplugin()
-plg.id    = 'ex_gramps'
-plg.name  = _('Gramps XML (family tree)')
-plg.name_accell  = _('Gramps _XML (family tree)')
-plg.description =  _('Gramps XML export is a complete archived XML backup of a' 
+plg.id = 'ex_gramps'
+plg.name = _('Gramps XML (family tree)')
+plg.name_accell = _('Gramps _XML (family tree)')
+plg.description = _('Gramps XML export is a complete archived XML backup of a'
                  ' Gramps family tree without the media object files.'
                  ' Suitable for backup purposes.')
 plg.version = '1.0'
@@ -146,7 +149,7 @@ plg.status = STABLE
 plg.fname = 'exportxml.py'
 plg.ptype = EXPORT
 plg.export_function = 'export_data'
-plg.export_options = 'WriterOptionBox'
+plg.export_options = 'WriterOptionBoxWithCompression'
 plg.export_options_title = _('Gramps XML export options')
 plg.extension = "gramps"
 
@@ -157,10 +160,10 @@ plg.extension = "gramps"
 #------------------------------------------------------------------------
 
 plg = newplugin()
-plg.id    = 'ex_vcal'
-plg.name  = _('vCalendar')
-plg.name_accell  = _('vC_alendar')
-plg.description =  _('vCalendar is used in many calendaring and PIM applications.')
+plg.id = 'ex_vcal'
+plg.name = _('vCalendar')
+plg.name_accell = _('vC_alendar')
+plg.description = _('vCalendar is used in many calendaring and PIM applications.')
 plg.version = '1.0'
 plg.gramps_target_version = MODULE_VERSION
 plg.status = STABLE
@@ -178,10 +181,10 @@ plg.extension = "vcs"
 #------------------------------------------------------------------------
 
 plg = newplugin()
-plg.id    = 'ex_vcard'
-plg.name  = _('vCard')
-plg.name_accell  = _('_vCard')
-plg.description =  _('vCard is used in many addressbook and pim applications.')
+plg.id = 'ex_vcard'
+plg.name = _('vCard')
+plg.name_accell = _('_vCard')
+plg.description = _('vCard is used in many addressbook and pim applications.')
 plg.version = '1.0'
 plg.gramps_target_version = MODULE_VERSION
 plg.status = STABLE
