@@ -4,6 +4,7 @@
 #
 # Copyright (C) 2000-2006  Donald N. Allingham
 # Copyright (C) 2009       Brian G. Matherly
+# Copyright (C) 2013       John Ralls
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,8 +26,6 @@
 # python modules
 #
 #------------------------------------------------------------------------
-# If you enable unicode_literals, some stuff needs fixing for win+py2, see #7162
-#from __future__ import print_function, unicode_literals
 import gettext
 import sys
 import os
@@ -34,6 +33,7 @@ import codecs
 import locale
 import collections
 import logging
+from binascii import hexlify
 
 LOG = logging.getLogger("." + __name__)
 LOG.propagate = True
@@ -65,6 +65,7 @@ if HAVE_ICU:
 # when a new translation is added.  Note the dummy _(): That's just to
 # get xgettext to include the string in gramps.pot; actual translation
 # is done in _get_language_string() below.
+# (The gramps officially-supported language list is ALL_LINGUAS in setup.py)
 _ = lambda x: x
 _LOCALE_NAMES = {
     'ar': ('Arabic_Saudi Arabia', '1256', _("Arabic")),
@@ -101,14 +102,23 @@ _LOCALE_NAMES = {
     'sk': ('Slovak_Slovakia', '1250', _("Slovak"),),
     'sl': ('Slovenian_Slovenia', '1250', _("Slovenian")),
     'sq': ('Albanian_Albania', '1250', _("Albanian")),
-    'sr': ('Serbian (Cyrillic)_Serbia', '1251', _("Serbian")),
+    'sr': ('Serbian(Cyrillic)_Serbia and Montenegro', '1251', _("Serbian")),
     'sv': ('Swedish_Sweden', '1252', _("Swedish")),
+    'ta': (None, None, _("Tamil")), # Windows has no codepage for Tamil
     'tr': ('Turkish_Turkey', '1254', _("Turkish")),
     'uk': ('Ukrainian_Ukraine', '1251', _("Ukrainian")),
-    'vi': ('Vietnamese_Viet Nam', '1258', _("Vietnamese")),
-    'zh_CN': ('Chinese_China', '936', _("Chinese Simplified")),
+    'vi': ('Vietnamese_Vietnam', '1258', _("Vietnamese")),
+    'zh_CN': ('Chinese_China', '936', _("Chinese (Simplified)")),
+    'zh_HK': ('Chinese_Hong Kong', '950', _("Chinese (Hong Kong)")),
+    'zh_TW': ('Chinese_Taiwan', '950', _("Chinese (Traditional)")),
     }
 
+# locales with right-to-left text
+_RTL_LOCALES = ('ar', 'he')
+
+# locales with less than 70% currently translated
+INCOMPLETE_TRANSLATIONS = ('ar', 'bg', 'he', 'ja', 'sq', 'ta', 'tr')
+
 def _check_mswin_locale(locale):
     msloc = None
     try:
@@ -119,6 +129,10 @@ def _check_mswin_locale(locale):
             msloc = _LOCALE_NAMES[locale[:2]][:2]
             locale = locale[:2]
         except KeyError:
+            #US English is the outlier, all other English locales want
+            #real English:
+            if locale[:2] == ('en') and locale[:5] != 'en_US':
+                return ('en_GB', '1252')
             return (None, None)
     return (locale, msloc)
 
@@ -126,15 +140,57 @@ def _check_mswin_locale_reverse(locale):
     for (loc, msloc) in _LOCALE_NAMES.items():
         if msloc and locale == msloc[0]:
             return (loc, msloc[1])
-
+    #US English is the outlier, all other English locales want real English:
+    if locale.startswith('English') and locale != 'English_United States':
+        return ('en_GB', '1252')
     return (None, None)
 
+def _check_gformat():
+    """
+    Some OS environments do not support the locale.nl_langinfo() method
+    of determing month names and other date related information.
+    """
+    try:
+        gformat = locale.nl_langinfo(locale.D_FMT).replace('%y','%Y')
+        # Gramps treats dates with '-' as ISO format, so replace separator
+        # on locale dates that use '-' to prevent confict
+        gformat = gformat.replace('-', '/')
+    except:
+        '''
+        Depending on the locale, the value returned for 20th Feb 2009
+        could be '20/2/2009', '20/02/2009', '20.2.2009', '20.02.2009',
+        '20-2-2009', '20-02-2009', '2009/02/20', '2009.02.20',
+        '2009-02-20', or '09-02-20' so to reduce the possible values to
+        test for, make sure both the month and the day are double digits,
+        preferably greater than 12 for human readablity
+        '''
+        import time
+        timestr = time.strftime('%x',(2005,10,25,1,1,1,1,1,1))
+
+        # Gramps treats dates with '-' as ISO format, so replace separator
+        # on locale dates that use '-' to prevent confict
+        timestr = timestr.replace('-', '/')
+
+        time2fmt_map = {'25/10/2005' : '%d/%m/%Y',
+                        '10/25/2005' : '%m/%d/%Y',
+                        '2005/10/25' : '%Y/%m/%d',
+                        '25.10.2005' : '%d.%m.%Y',
+                        '10.25.2005' : '%m.%d.%Y',
+                        '2005.10.25' : '%Y.%m.%d',
+                       }
+
+        try:
+            gformat = time2fmt_map[timestr]
+        except KeyError:
+            gformat = '%d/%m/%Y'  # default value
+    return gformat
+
 #------------------------------------------------------------------------
 #
 # GrampsLocale Class
 #
 #------------------------------------------------------------------------
-class GrampsLocale(object):
+class GrampsLocale:
     """
     Encapsulate a locale.  This class is a sort-of-singleton: The
     first instance created will query the environment and OSX defaults
@@ -283,6 +339,9 @@ class GrampsLocale(object):
             if not locale[0]:
                 return False
             lang = self.check_available_translations(locale[0])
+            if not lang and locale[0].startswith('en'):
+                locale = ('en_GB', 'UTF-8')
+                lang = 'en_GB'
             if not lang:
                 return False
             self.lang = locale[0]
@@ -293,13 +352,12 @@ class GrampsLocale(object):
         _failure = False
         try:
             locale.setlocale(locale.LC_ALL, '')
-            if not _check_locale(locale.getlocale()):
-                if not _check_locale(locale.getdefaultlocale()):
-                    LOG.debug("Usable locale not found, localization settings ignored.");
-                    self.lang = 'C'
-                    self.encoding = 'ascii'
-                    self.language = ['en']
-                    _failure = True
+            if not _check_locale(locale.getdefaultlocale(envvars=('LC_ALL', 'LANG', 'LANGUAGE'))):
+                LOG.debug("Usable locale not found, localization settings ignored.");
+                self.lang = 'C'
+                self.encoding = 'ascii'
+                self.language = ['en']
+                _failure = True
 
         except locale.Error as err:
             LOG.debug("Locale error %s, localization settings ignored.",
@@ -318,18 +376,6 @@ class GrampsLocale(object):
             else:
                 LOG.debug("No translation for LC_MESSAGES locale %s", loc)
 
-        # $LANGUAGE overrides $LANG, $LC_MESSAGES
-        if "LANGUAGE" in os.environ:
-            language = [x for x in [self.check_available_translations(l)
-                                    for l in os.environ["LANGUAGE"].split(":")]
-                            if x]
-            if language:
-                self.language = language
-                if not self.lang.startswith(self.language[0]):
-                    LOG.debug("Overiding locale setting %s with LANGUAGE setting %s", self.lang, self.language[0])
-            elif _failure:
-                LOG.warning("No valid locale settings found, using US English")
-
         if HAVE_ICU:
             self.calendar = locale.getlocale(locale.LC_TIME)[0] or self.lang[:5]
             self.collation = locale.getlocale(locale.LC_COLLATE)[0] or self.lang[:5]
@@ -361,6 +407,23 @@ class GrampsLocale(object):
         else:
             self.currency = self.lang
 
+        # $LANGUAGE overrides $LANG, $LC_MESSAGES
+        if "LANGUAGE" in os.environ:
+            language = [x for x in [self.check_available_translations(l)
+                                    for l in os.environ["LANGUAGE"].split(":")]
+                            if x]
+            if language:
+                self.language = language
+                if not self.lang.startswith(self.language[0]):
+                    LOG.debug("Overiding locale setting '%s' with LANGUAGE setting '%s'", self.lang, self.language[0])
+                    self.lang = self.calendar = self.language[0]
+            elif _failure:
+                LOG.warning("No valid locale settings found, using US English")
+
+        if __debug__:
+            LOG.debug("The locale tformat for '%s' is '%s'",
+                      self.lang, _check_gformat())
+
     def _win_bindtextdomain(self, localedomain, localedir):
         """
         Help routine for loading and setting up libintl attributes
@@ -421,24 +484,6 @@ class GrampsLocale(object):
             self.encoding = (locale.getpreferredencoding()
                              or sys.getdefaultencoding())
         LOG.debug("Setting encoding to %s", self.encoding)
-#Ensure that output is encoded correctly to stdout and stderr. This is
-#much less cumbersome and error-prone than encoding individual outputs
-#and better handles the differences between Python 2 and Python 3:
-        try:
-            _encoding = sys.stdout.encoding or sys.getdefaultencoding()
-        except:
-            _encoding = "UTF-8"
-        if sys.version_info[0] < 3:
-            sys.stdout = codecs.getwriter(_encoding)(sys.stdout,
-                                                     'backslashreplace')
-            sys.stderr = codecs.getwriter(_encoding)(sys.stderr,
-                                                     'backslashreplace')
-        else:
-            sys.stdout = codecs.getwriter(_encoding)(sys.stdout.detach(),
-                                                     'backslashreplace')
-            sys.stderr = codecs.getwriter(_encoding)(sys.stderr.detach(),
-                                                     'backslashreplace')
-
 
         # Make sure that self.lang and self.language are reflected
         # back into the environment for Gtk to use when its
@@ -485,18 +530,25 @@ class GrampsLocale(object):
                 self._win_bindtextdomain(self.localedomain.encode('utf-8'),
                                          self.localedir.encode('utf-8'))
 
+        self.rtl_locale = False
+        if self.language[0] in _RTL_LOCALES:
+            self.rtl_locale = True # right-to-left
+
     def _init_secondary_locale(self):
         """
         Init a secondary locale. Secondary locales are used to provide
         an alternate localization to the one used for the UI; for
         example, some reports offer the option to use a different
         language.
+
+        This GrampsLocale class does no caching of the secondary locale.
+        If any caching is desired it must be done externally.
         """
         if not self.localedir:
             LOG.warning("No Localedir provided, unable to find translations")
 
         if not self.localedomain:
-            if _firstlocaledomain:
+            if _firstlocaledomain: # TODO this variable is nowhere else
                 self.localedomain = _first.localedomain
             else:
                 self.localedomain = "gramps"
@@ -514,7 +566,11 @@ class GrampsLocale(object):
         if not self.language and _first.language:
             self.language = _first.language
 
-        self.calendar = self.collation = self.lang
+        self.numeric = self.currency = self.calendar = self.collation = self.lang
+
+        self.rtl_locale = False
+        if self.language[0] in _RTL_LOCALES:
+            self.rtl_locale = True # right-to-left
 
     def __init__(self, localedir=None, lang=None, domain=None, languages=None):
         """
@@ -539,8 +595,8 @@ class GrampsLocale(object):
               os.path.exists(os.path.abspath(_first.localedir))):
             self.localedir = _first.localedir
         else:
-            LOG.warn('Missing or invalid localedir %s; no translations will be available.', repr(localedir))
-
+            LOG.warning('Missing or invalid localedir %s; no translations'
+                        ' will be available.', repr(localedir))
         self.lang = lang
         self.localedomain = domain or 'gramps'
         if languages:
@@ -660,24 +716,24 @@ class GrampsLocale(object):
         if self._dd:
             return self._dd
 
-        from gramps.gen.config import config
+        from ..config import config
         try:
             val = config.get('preferences.date-format')
         except AttributeError:
             val = 0;
 
-        from gramps.gen.datehandler import LANG_TO_DISPLAY as displayers
+        from ..datehandler import LANG_TO_DISPLAY as displayers
         _first = self._GrampsLocale__first_instance
         if self.calendar in displayers:
             self._dd = displayers[self.calendar](val)
         elif self.calendar[:2] in displayers:
             self._dd = displayers[self.calendar[:2]](val)
         elif self != _first and _first.calendar in displayers:
-            self._dd = displayers[_first.calendar](val)
+            self._dd = displayers[_first.calendar](val, blocale=self)
         elif self != _first and _first.calendar[:2] in displayers:
-            self._dd = displayers[_first.calendar[:2]](val)
+            self._dd = displayers[_first.calendar[:2]](val, blocale=self)
         else:
-            self._dd = displayers['C'](val)
+            self._dd = displayers['C'](val, blocale=self)
 
         return self._dd
 
@@ -694,7 +750,7 @@ class GrampsLocale(object):
         if self._dp:
             return self._dp
 
-        from gramps.gen.datehandler import LANG_TO_PARSER as parsers
+        from ..datehandler import LANG_TO_PARSER as parsers
         _first = self._GrampsLocale__first_instance
         if self.calendar in parsers:
             self._dp = parsers[self.calendar]()
@@ -813,9 +869,11 @@ class GrampsLocale(object):
 
         if locale[:5] in self.languages:
             return locale[:5]
+        #US English is the outlier, all other English locales want real English:
+        if locale[:2] == 'en' and locale[:5] != 'en_US':
+            return 'en_GB'
         if locale[:2] in self.languages:
             return locale[:2]
-
         return None
 
     def get_language_dict(self):
@@ -846,12 +904,14 @@ class GrampsLocale(object):
             return _("the repository")
         elif objclass == "note":
             return _("the note")
-        elif objclass in ["media", "mediaobject"]:
+        elif objclass == "media":
             return _("the media")
         elif objclass == "source":
             return _("the source")
         elif objclass == "filter":
             return _("the filter")
+        elif objclass == "citation":
+            return _("the citation")
         else:
             return _("See details")
 
@@ -861,21 +921,26 @@ class GrampsLocale(object):
         """
 
         if HAVE_ICU and self.collator:
-            #ICU can digest strings and unicode
-            return self.collator.getCollationKey(string).getByteArray()
+            # ICU can digest strings and unicode
+            # Use hexlify() as to make a consistent string, fixing bug #10077
+            return hexlify(self.collator.getCollationKey(string).getByteArray()).decode()
         else:
-            if sys.version_info[0] < 3 and isinstance(string, unicode):
-                string = string.encode("utf-8", "replace")
-            if sys.version_info[0] >= 3 and isinstance(string, bytes):
+            if isinstance(string, bytes):
                 string = string.decode("utf-8", "replace")
             try:
                 key = locale.strxfrm(string)
             except Exception as err:
-                LOG.warn("Failed to obtain key for %s because %s",
+                LOG.warning("Failed to obtain key for %s because %s",
                          self.collation, str(err))
                 return string
             return key
 
+    def get_collation(self):
+        """
+        Return the collation without any character encoding.
+        """
+        return self.collation.split('.')[0]
+
     def strcoll(self, string1, string2):
         """
         Given two localized strings, compare them and return -1 if
@@ -908,7 +973,7 @@ class GrampsLocale(object):
         :returns: The name as text in the proper language.
         :rtype: unicode
         """
-        from gramps.gen.lib.grampstype import GrampsType
+        from ..lib.grampstype import GrampsType
         return GrampsType.xml_str(name)
 
     def format(self, format, val, grouping=False, monetary=False):
@@ -953,13 +1018,7 @@ class GrampsLocale(object):
 # Translations Classes
 #
 #-------------------------------------------------------------------------
-if sys.version_info < (3,0):
-    _LexemeBaseStr = unicode 
-    _isstring = lambda s: isinstance(s, basestring)
-else:
-    _LexemeBaseStr = str
-    _isstring = lambda s: isinstance(s, str)
-class Lexeme(_LexemeBaseStr):
+class Lexeme(str):
     r"""
     Created with :meth:`~GrampsTranslations.lexgettext`
 
@@ -993,8 +1052,8 @@ class Lexeme(_LexemeBaseStr):
     Prints out::
 
         In English locale:
-            CHRISTMAS 
-            Christmas is celebrated in December 
+            CHRISTMAS
+            Christmas is celebrated in December
             Merry Christmas!
 
         In Russian locale:
@@ -1008,17 +1067,17 @@ class Lexeme(_LexemeBaseStr):
     These forms are accessible under dictionary keys for each form.
     The names of the forms are language-specific. They are assigned
     by the human translator of the corresponding language (in XX.po)
-    as in the example above, 
-    see :meth:`~GrampsTranslations.lexgettext` docs 
+    as in the example above,
+    see :meth:`~GrampsTranslations.lexgettext` docs
     for more info.
 
     The translated format string can then refer to a specific form
-    of the lexeme using ``.``:attr:`~Lexeme.f` and square brackets: 
+    of the lexeme using ``.``:attr:`~Lexeme.f` and square brackets:
     ``{holiday.f[GENITIVE]}``
     expects holiday to be a Lexeme which has a form ``'GENITIVE'`` in it.
 
     An instance of Lexeme can also be used as a regular unicode string.
-    In this case, the work will be delegated to the string for the very 
+    In this case, the work will be delegated to the string for the very
     first form provided in the translated string. In the example above,
     ``{holiday}`` in the translated string will expand to the Russian
     nominative form for Christmas, and ``xmas.upper()`` will produce
@@ -1026,9 +1085,9 @@ class Lexeme(_LexemeBaseStr):
 
     .. rubric:: Motivation
 
-    Lexeme is the term used in linguistics for the set of forms taken 
+    Lexeme is the term used in linguistics for the set of forms taken
     by a particular word, e.g. cases for a noun or tenses for a verb.
-    
+
     Gramps often needs to compose sentences from several blocks of
     text and single words, often by using python string formatting.
 
@@ -1041,7 +1100,7 @@ class Lexeme(_LexemeBaseStr):
                  enddate_month = m2,
                  enddate_year = y2)
 
-    To make such text translatable, the arguments injected into 
+    To make such text translatable, the arguments injected into
     format string need to bear all the linguistical information
     on how to plug them into a sentence, i.e., the forms, depending
     on the linguistic context of where the argument appears.
@@ -1050,18 +1109,18 @@ class Lexeme(_LexemeBaseStr):
 
     On the other hand, for languages where there is no linguistic
     variation in such sentences, the code needs not to be aware of
-    the underlying :class:`~Lexeme` complexity; 
-    and so they can be processed just like simple strings 
+    the underlying :class:`~Lexeme` complexity;
+    and so they can be processed just like simple strings
     both when passed around in the code and when formatted.
     """
 
     def __new__(cls, iterable, *args, **kwargs):
-        if _isstring(iterable):
-            newobj = _LexemeBaseStr.__new__(cls, iterable, *args, **kwargs)
+        if isinstance(iterable, str):
+            newobj = str.__new__(cls, iterable, *args, **kwargs)
         else:
             od = collections.OrderedDict(iterable)
             l = list(od.values()) or [""]
-            newobj = _LexemeBaseStr.__new__(cls, l[0], *args, **kwargs) 
+            newobj = str.__new__(cls, l[0], *args, **kwargs)
             newobj._forms = od
         return newobj
 
@@ -1069,7 +1128,7 @@ class Lexeme(_LexemeBaseStr):
         """All lexeme forms, in the same order as given upon construction.
         The first one returned is the default form, which is used when the
         Lexeme instance is used in lieu of a string object.
-        
+
         Same as ``f.values()``"""
         return self._forms.values()
 
@@ -1102,10 +1161,7 @@ class GrampsTranslations(gettext.GNUTranslations):
         # and that's not what we want.
         if len(msgid.strip()) == 0:
             return msgid
-        if sys.version_info[0] < 3:
-            return gettext.GNUTranslations.ugettext(self, msgid)
-        else:
-            return gettext.GNUTranslations.gettext(self, msgid)
+        return gettext.GNUTranslations.gettext(self, msgid)
 
     def ngettext(self, singular, plural, num):
         """
@@ -1123,12 +1179,7 @@ class GrampsTranslations(gettext.GNUTranslations):
         :returns: Translation or the original.
         :rtype: unicode
         """
-        if sys.version_info[0] < 3:
-            return gettext.GNUTranslations.ungettext(self, singular,
-                                                     plural, num)
-        else:
-            return gettext.GNUTranslations.ngettext(self, singular,
-                                                    plural, num)
+        return gettext.GNUTranslations.ngettext(self, singular, plural, num)
 
     def sgettext(self, msgid, sep='|'):
         """
@@ -1158,7 +1209,7 @@ class GrampsTranslations(gettext.GNUTranslations):
         stripping the '|'-separated context using :meth:`~sgettext`
 
         The *resulting* message provided by the translator
-        is supposed to be '|'-separated as well. 
+        is supposed to be '|'-separated as well.
         The possible formats are either (1) a single string
         for a language with no inflections, or (2) a list of
         <inflection name>=<inflected form>, separated with '|'.
