@@ -22,8 +22,6 @@
 Utility functions that depend on GUI components or for GUI components
 """
 
-from __future__ import print_function, division
-
 #-------------------------------------------------------------------------
 #
 # Standard python modules
@@ -33,8 +31,6 @@ import os
 import sys
 import subprocess
 import threading
-from gramps.gen.const import GRAMPS_LOCALE as glocale
-_ = glocale.translation.gettext
 # gtk is not included here, because this file is currently imported
 # by code that needs to run without the DISPLAY variable (eg, in
 # the cli only).
@@ -44,14 +40,20 @@ _ = glocale.translation.gettext
 # GNOME/GTK
 #
 #-------------------------------------------------------------------------
+import gi
+gi.require_version('PangoCairo', '1.0')
 from gi.repository import PangoCairo
 from gi.repository import GLib
+from gi.repository import Gdk
 
 #-------------------------------------------------------------------------
 #
 # Gramps modules
 #
 #-------------------------------------------------------------------------
+from gramps.gen.const import GRAMPS_LOCALE as glocale
+_ = glocale.translation.gettext
+from gramps.gen.lib import EventType, EventRoleType, FamilyRelType
 from gramps.gen.lib.person import Person
 from gramps.gen.constfunc import has_display, is_quartz, mac, win
 from gramps.gen.config import config
@@ -64,7 +66,7 @@ from gramps.gen.errors import WindowActiveError
 #
 #-------------------------------------------------------------------------
 
-class CLIVbox():
+class CLIVbox:
     """
     Command-line interface vbox, to keep compatible with Dialog.
     """
@@ -103,7 +105,7 @@ class CLIDialog:
 #
 #-------------------------------------------------------------------------
 
-class ProgressMeter(object):
+class ProgressMeter:
     """
     Progress meter class for Gramps.
 
@@ -152,6 +154,12 @@ class ProgressMeter(object):
         self.__dialog.vbox.set_spacing(10)
         self.__dialog.vbox.set_border_width(24)
         self.__dialog.set_size_request(400, 125)
+        if not parent:  # if we don't have an explicit parent, try to find one
+            for win in Gtk.Window.list_toplevels():
+                if win.is_active():
+                    parent = win
+                    break
+        # if we still don't have a parent, give up
         if parent:
             self.__dialog.set_transient_for(parent)
             self.__dialog.set_modal(True)
@@ -169,7 +177,7 @@ class ProgressMeter(object):
 
         if self.__can_cancel:
             self.__dialog.set_size_request(350, 170)
-            self.__cancel_button = Gtk.Button(stock=Gtk.STOCK_CANCEL)
+            self.__cancel_button = Gtk.Button.new_with_mnemonic(_('_Cancel'))
             self.__cancel_button.connect('clicked', self.__cancel_callback)
             self.__dialog.vbox.add(self.__cancel_button)
 
@@ -180,10 +188,10 @@ class ProgressMeter(object):
             text.set_border_width(6)
             text.set_editable(False)
             self.message_area = text
-            area.add_with_viewport(text)
+            area.add(text)
             area.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
             self.__dialog.vbox.add(area)
-            self.message_area_ok = Gtk.Button(stock=Gtk.STOCK_OK)
+            self.message_area_ok = Gtk.Button.new_with_mnemonic(_('_OK'))
             self.message_area_ok.connect("clicked", self.close)
             self.message_area_ok.set_sensitive(False)
             self.__dialog.vbox.pack_start(self.message_area_ok, expand=False, fill=False)
@@ -300,13 +308,14 @@ class ProgressMeter(object):
         WarningDialog(
             _("Attempt to force closing the dialog"),
             _("Please do not force closing this important dialog."),
-            self.__dialog)
+            parent=self.__dialog)
         return True
 
     def close(self, widget=None):
         """
         Close the progress meter
         """
+        del self.__cancel_callback
         self.__dialog.destroy()
 
 #-------------------------------------------------------------------------
@@ -315,9 +324,9 @@ class ProgressMeter(object):
 #
 #-------------------------------------------------------------------------
 
-class SystemFonts(object):
+class SystemFonts:
     """
-    Define fonts available to Gramps    
+    Define fonts available to Gramps
 
     This is a workaround for bug which prevents the list_families method
     being called more than once.
@@ -325,7 +334,7 @@ class SystemFonts(object):
     The bug is described here: https://bugzilla.gnome.org/show_bug.cgi?id=679654
 
     This code generates a warning:
-    /usr/local/lib/python2.7/site-packages/gi/types.py:47: 
+    /usr/local/lib/python2.7/site-packages/gi/types.py:47:
     Warning: g_value_get_object: assertion `G_VALUE_HOLDS_OBJECT (value)' failed
 
     To get a list of fonts, instantiate this class and call
@@ -357,7 +366,7 @@ class SystemFonts(object):
 #
 #
 #-------------------------------------------------------------------------
-def display_error_dialog (index, errorstrings):
+def display_error_dialog (index, errorstrings, uistate=None):
     """
     Display a message box for errors resulting from xdg-open/open
     """
@@ -372,7 +381,8 @@ def display_error_dialog (index, errorstrings):
         else:
             error = errorstrings
 
-    ErrorDialog(_("Error from external program"), error)
+    ErrorDialog(_("Error from external program"),
+                error, parent=uistate.window)
 
 def poll_external (args):
     """
@@ -387,16 +397,16 @@ def poll_external (args):
     :return: bool returned to timeout_add_seconds: should this function be
              called again?
     """
-    (proc, errorstrings) = args
+    (proc, errorstrings, uistate) = args
     resp = proc.poll()
     if resp is None:
         return True
 
     if resp != 0:
-        display_error_dialog(resp, errorstrings)
+        display_error_dialog(resp, errorstrings, uistate)
     return False
 
-def open_file_with_default_application(path):
+def open_file_with_default_application(path, uistate):
     """
     Launch a program to open an arbitrary file. The file will be opened using
     whatever program is configured on the host as the default program for that
@@ -412,14 +422,16 @@ def open_file_with_default_application(path):
 
     norm_path = os.path.normpath(path)
     if not os.path.exists(norm_path):
-        display_error_dialog(0, _("File does not exist"))
+        display_error_dialog(0, _("File %s does not exist") % norm_path,
+                             uistate)
         return
 
     if win():
         try:
             os.startfile(norm_path)
         except WindowsError as msg:
-            display_error_dialog(0, str(msg))
+            display_error_dialog(0, str(msg),
+                             uistate)
 
         return
 
@@ -435,7 +447,7 @@ def open_file_with_default_application(path):
     proc = subprocess.Popen([utility, norm_path], stderr=subprocess.STDOUT)
 
     from gi.repository import GLib
-    GLib.timeout_add_seconds(1, poll_external, (proc, errstrings))
+    GLib.timeout_add_seconds(1, poll_external, (proc, errstrings, uistate))
     return
 
 def process_pending_events(max_count=10):
@@ -457,18 +469,41 @@ def process_pending_events(max_count=10):
 
 def is_right_click(event):
     """
-    Returns True if the event is a button-3 or equivalent
+    Returns True if the event is to open the context menu.
     """
     from gi.repository import Gdk
+    if Gdk.Event.triggers_context_menu(event):
+        return True
 
-    if event.type == Gdk.EventType.BUTTON_PRESS:
-        if is_quartz():
-            if (event.button == 3
-                or (event.button == 1 and event.get_state() & Gdk.ModifierType.CONTROL_MASK)):
-                return True
+def color_graph_family(family, dbstate):
+    """
+    :return: based on the config the color for graph family node in hex
+    :rtype: tuple (hex color fill, hex color border)
+    """
+    scheme = config.get('colors.scheme')
+    for event_ref in family.get_event_ref_list():
+        event = dbstate.db.get_event_from_handle(event_ref.ref)
+        if (event.type == EventType.DIVORCE and
+                event_ref.get_role() in (EventRoleType.FAMILY,
+                                         EventRoleType.PRIMARY)):
+            return (config.get('colors.family-divorced')[scheme],
+                    config.get('colors.border-family-divorced')[scheme])
 
-        if event.button == 3:
-            return True
+    fam_rel_type = family.get_relationship()
+
+    family_color = config.get('colors.family')[scheme]
+    border_color = config.get('colors.border-family')[scheme]
+
+    if fam_rel_type == FamilyRelType.MARRIED:
+        family_color = config.get('colors.family-married')[scheme]
+    elif fam_rel_type == FamilyRelType.UNMARRIED:
+        family_color = config.get('colors.family-unmarried')[scheme]
+    elif fam_rel_type == FamilyRelType.CIVIL_UNION:
+        family_color = config.get('colors.family-civil-union')[scheme]
+    elif fam_rel_type == FamilyRelType.UNKNOWN:
+        family_color = config.get('colors.family-unknown')[scheme]
+
+    return (family_color, border_color)
 
 def color_graph_box(alive=False, gender=Person.MALE):
     """
@@ -476,27 +511,28 @@ def color_graph_box(alive=False, gender=Person.MALE):
              If gender is None, an empty box is assumed
     :rtype: tuple (hex color fill, hex color border)
     """
+    scheme = config.get('colors.scheme')
     if gender == Person.MALE:
         if alive:
-            return (config.get('preferences.color-gender-male-alive'),
-                    config.get('preferences.bordercolor-gender-male-alive'))
+            return (config.get('colors.male-alive')[scheme],
+                    config.get('colors.border-male-alive')[scheme])
         else:
-            return (config.get('preferences.color-gender-male-death'),
-                    config.get('preferences.bordercolor-gender-male-death'))
+            return (config.get('colors.male-dead')[scheme],
+                    config.get('colors.border-male-dead')[scheme])
     elif gender == Person.FEMALE:
         if alive:
-            return (config.get('preferences.color-gender-female-alive'),
-                    config.get('preferences.bordercolor-gender-female-alive'))
+            return (config.get('colors.female-alive')[scheme],
+                    config.get('colors.border-female-alive')[scheme])
         else:
-            return (config.get('preferences.color-gender-female-death'),
-                    config.get('preferences.bordercolor-gender-female-death'))
+            return (config.get('colors.female-dead')[scheme],
+                    config.get('colors.border-female-dead')[scheme])
     elif gender == Person.UNKNOWN:
         if alive:
-            return (config.get('preferences.color-gender-unknown-alive'),
-                    config.get('preferences.bordercolor-gender-unknown-alive'))
+            return (config.get('colors.unknown-alive')[scheme],
+                    config.get('colors.border-unknown-alive')[scheme])
         else:
-            return (config.get('preferences.color-gender-unknown-death'),
-                    config.get('preferences.bordercolor-gender-unknown-death'))
+            return (config.get('colors.unknown-dead')[scheme],
+                    config.get('colors.border-unknown-dead')[scheme])
     #empty box, no gender
     return ('#d2d6ce', '#000000')
 ##    print 'male alive', rgb_to_hex((185/256.0, 207/256.0, 231/256.0))
@@ -540,21 +576,20 @@ def rgb_to_hex(rgb):
         rgbint = (int(rgb[0] * 255), int(rgb[1] * 255), int(rgb[2] * 255))
         return '#%02x%02x%02x' % rgbint
 
-def color_to_hex(color):
-    """Convert Gdk.Color to hex string."""
-    hexstring = ""
-    for col in 'red', 'green', 'blue':
-        hexfrag = hex(getattr(color, col) // (16 * 16)).split("x")[1]
-        if len(hexfrag) < 2:
-            hexfrag = "0" + hexfrag
-        hexstring += hexfrag
-    return '#' + hexstring
-    
-def hex_to_color(hex):
-    """Convert hex string to Gdk.Color."""
-    from gi.repository import Gdk
-    color = Gdk.color_parse(hex)
-    return color
+def get_link_color(context):
+    """
+    Find the link color for the current theme.
+    """
+    from gi.repository import Gtk
+
+    if Gtk.get_minor_version() > 11:
+        col = context.get_color(Gtk.StateFlags.LINK)
+    else:
+        found, col = context.lookup_color('link_color')
+        if not found:
+            col.parse('blue')
+
+    return rgb_to_hex((col.red, col.green, col.blue))
 
 def edit_object(dbstate, uistate, reftype, ref):
     """
@@ -600,19 +635,20 @@ def edit_object(dbstate, uistate, reftype, ref):
                              "To edit the citation, close the source "
                              "editor and open an editor for the citation "
                              "alone")
-            
+
             from .dialog import WarningDialog
             WarningDialog(_("Cannot open new citation editor"),
-                          blocked_text)
+                          blocked_text,
+                          parent=uistate.window)
     elif reftype == 'Place':
         try:
             place = dbstate.db.get_place_from_handle(ref)
             EditPlace(dbstate, uistate, [], place)
         except WindowActiveError:
             pass
-    elif reftype == 'MediaObject':
+    elif reftype == 'Media':
         try:
-            obj = dbstate.db.get_object_from_handle(ref)
+            obj = dbstate.db.get_media_from_handle(ref)
             EditMedia(dbstate, uistate, [], obj)
         except WindowActiveError:
             pass
@@ -647,3 +683,67 @@ class AvailableUpdates(threading.Thread):
         self.addon_update_list = available_updates()
         if len(self.addon_update_list) > 0:
             GLib.idle_add(self.emit_update_available)
+
+def model_to_text(model, cols=None, treeiter=None, indent="",
+                  level=None, sep=", "):
+    """
+    Given a model, return the text from the rows as a string.
+      model - the actual model
+      cols - a list representing the columns to get, or None for all
+      treeiter - (optional), initially, the first iterator
+      ident - the current indent level text
+      level - use None for no level indicator, or number (eg, 1)
+      sep - separating text between columns
+    """
+    text = ""
+    if treeiter is None:
+        treeiter = model.get_iter_first()
+    while treeiter is not None:
+        if cols is None:
+            items = sep.join([str(item) for item in model[treeiter][:]])
+        else:
+            items = sep.join([str(model[treeiter][col]) for col in cols])
+        if level is not None:
+            text += (indent + str(level) + ". " + items + "\n")
+        else:
+            text += (indent + items + "\n")
+        if model.iter_has_child(treeiter):
+            childiter = model.iter_children(treeiter)
+            if level is not None:
+                text += model_to_text(model, cols, childiter, indent + (" " * 4),
+                                      level + 1, sep)
+            else:
+                text += model_to_text(model, cols, childiter, indent + (" " * 4),
+                                      sep=sep)
+        treeiter = model.iter_next(treeiter)
+    return text
+
+def text_to_clipboard(text):
+    """
+    Put any text into the clipboard
+    """
+    from gi.repository import Gdk
+    from gi.repository import Gtk
+    clipboard = Gtk.Clipboard.get_for_display(Gdk.Display.get_default(),
+                                              Gdk.SELECTION_CLIPBOARD)
+    clipboard.set_text(text, -1)
+
+def match_primary_mask(test_mask, addl_mask=0):
+    """
+    Return True if test_mask fully matches all bits of
+    GdkModifierIntent.PRIMARY_ACCELERATOR and addl_mask, False
+    otherwise.
+    """
+    keymap = Gdk.Keymap.get_default()
+    primary = keymap.get_modifier_mask(Gdk.ModifierIntent.PRIMARY_ACCELERATOR)
+    return ((test_mask & (primary | addl_mask)) == (primary | addl_mask))
+
+def no_match_primary_mask(test_mask, addl_mask=0):
+    """
+    Return False if test_mask matches any bit of
+    GdkModifierIntent.PRIMARY_ACCELERATOR or addl_mask, True
+    otherwise.
+    """
+    keymap = Gdk.Keymap.get_default()
+    primary = keymap.get_modifier_mask(Gdk.ModifierIntent.PRIMARY_ACCELERATOR)
+    return (test_mask & (primary | addl_mask)) == 0
