@@ -9,7 +9,7 @@
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful, 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
@@ -28,6 +28,8 @@ Provide the base classes for GRAMPS' DataView classes
 # python modules
 #
 #----------------------------------------------------------------
+from abc import abstractmethod
+import html
 import logging
 
 _LOG = logging.getLogger('.navigationview')
@@ -42,32 +44,23 @@ from gi.repository import Gtk
 
 #----------------------------------------------------------------
 #
-# GRAMPS 
+# Gramps
 #
 #----------------------------------------------------------------
-from .pageview import PageView
-from ..actiongroup import ActionGroup
 from gramps.gen.const import GRAMPS_LOCALE as glocale
 _ = glocale.translation.sgettext
+from .pageview import PageView
+from ..uimanager import ActionGroup
 from gramps.gen.utils.db import navigation_label
 from gramps.gen.constfunc import mod_key
+from ..utils import match_primary_mask
 
 DISABLED = -1
 MRU_SIZE = 10
 
-MRU_TOP = [
-    '<ui>'
-    '<menubar name="MenuBar">'
-    '<menu action="GoMenu">'
-    '<placeholder name="CommonHistory">'
-    ]
+MRU_TOP = '<section id="CommonHistory">'
+MRU_BTM = '</section>'
 
-MRU_BTM = [
-    '</placeholder>'
-    '</menu>'
-    '</menubar>'
-    '</ui>'
-    ]
 #------------------------------------------------------------------------------
 #
 # NavigationView
@@ -79,7 +72,7 @@ class NavigationView(PageView):
     navigation functionalilty. Views that need bookmarks and forward/backward
     should derive from this class.
     """
-    
+
     def __init__(self, title, pdata, state, uistate, bm_type, nav_group):
         PageView.__init__(self, title, pdata, state, uistate)
         self.bookmarks = bm_type(self.dbstate, self.uistate, self.change_active)
@@ -92,14 +85,15 @@ class NavigationView(PageView):
         self.mru_signal = None
         self.nav_group = nav_group
         self.mru_active = DISABLED
+        self.uimanager = uistate.uimanager
 
         self.uistate.register(state, self.navigation_type(), self.nav_group)
 
-    
+
     def navigation_type(self):
         """
         Indictates the navigation type. Navigation type can be the string
-        name of any of the primary Objects. A History object will be 
+        name of any of the primary Objects. A History object will be
         created for it, see DisplayState.History
         """
         return None
@@ -111,40 +105,45 @@ class NavigationView(PageView):
         PageView.define_actions(self)
         self.bookmark_actions()
         self.navigation_actions()
-        
+
     def disable_action_group(self):
         """
-        Normally, this would not be overridden from the base class. However, 
+        Normally, this would not be overridden from the base class. However,
         in this case, we have additional action groups that need to be
         handled correctly.
         """
         PageView.disable_action_group(self)
-        
-        self.fwd_action.set_visible(False)
-        self.back_action.set_visible(False)
+
+        self.uimanager.set_actions_visible(self.fwd_action, False)
+        self.uimanager.set_actions_visible(self.back_action, False)
 
     def enable_action_group(self, obj):
         """
-        Normally, this would not be overridden from the base class. However, 
+        Normally, this would not be overridden from the base class. However,
         in this case, we have additional action groups that need to be
         handled correctly.
         """
         PageView.enable_action_group(self, obj)
-        
-        self.fwd_action.set_visible(True)
-        self.back_action.set_visible(True)
+
+        self.uimanager.set_actions_visible(self.fwd_action, True)
+        self.uimanager.set_actions_visible(self.back_action, True)
         hobj = self.get_history()
-        self.fwd_action.set_sensitive(not hobj.at_end())
-        self.back_action.set_sensitive(not hobj.at_front())
+        self.uimanager.set_actions_sensitive(self.fwd_action,
+                                             not hobj.at_end())
+        self.uimanager.set_actions_sensitive(self.back_action,
+                                             not hobj.at_front())
 
     def change_page(self):
         """
         Called when the page changes.
         """
         hobj = self.get_history()
-        self.fwd_action.set_sensitive(not hobj.at_end())
-        self.back_action.set_sensitive(not hobj.at_front())
-        self.other_action.set_sensitive(not self.dbstate.db.readonly)
+        self.uimanager.set_actions_sensitive(self.fwd_action,
+                                             not hobj.at_end())
+        self.uimanager.set_actions_sensitive(self.back_action,
+                                             not hobj.at_front())
+        self.uimanager.set_actions_sensitive(self.other_action,
+                                             not self.dbstate.db.readonly)
         self.uistate.modify_statusbar(self.dbstate)
 
     def set_active(self):
@@ -153,12 +152,12 @@ class NavigationView(PageView):
         """
         PageView.set_active(self)
         self.bookmarks.display()
-        
+
         hobj = self.get_history()
         self.active_signal = hobj.connect('active-changed', self.goto_active)
         self.mru_signal = hobj.connect('mru-changed', self.update_mru_menu)
-        self.update_mru_menu(hobj.mru)
-            
+        self.update_mru_menu(hobj.mru, update_menu=False)
+
         self.goto_active(None)
 
     def set_inactive(self):
@@ -178,7 +177,7 @@ class NavigationView(PageView):
         Return the navigation group.
         """
         return self.nav_group
-        
+
     def get_history(self):
         """
         Return the history object.
@@ -195,10 +194,12 @@ class NavigationView(PageView):
                                                 self.navigation_group())
         if active_handle:
             self.goto_handle(active_handle)
-            
+
         hobj = self.get_history()
-        self.fwd_action.set_sensitive(not hobj.at_end())
-        self.back_action.set_sensitive(not hobj.at_front())
+        self.uimanager.set_actions_sensitive(self.fwd_action,
+                                             not hobj.at_end())
+        self.uimanager.set_actions_sensitive(self.back_action,
+                                             not hobj.at_front())
 
     def get_active(self):
         """
@@ -207,27 +208,27 @@ class NavigationView(PageView):
         hobj = self.uistate.get_history(self.navigation_type(),
                                         self.navigation_group())
         return hobj.present()
-        
+
     def change_active(self, handle):
         """
         Changes the active object.
         """
         hobj = self.get_history()
         if handle and not hobj.lock and not (handle == hobj.present()):
-            hobj.push(handle)            
+            hobj.push(handle)
 
+    @abstractmethod
     def goto_handle(self, handle):
         """
         Needs to be implemented by classes derived from this.
         Used to move to the given handle.
         """
-        raise NotImplementedError
 
     def selected_handles(self):
         """
         Return the active person's handle in a list. Used for
         compatibility with those list views that can return multiply
-        selected items. 
+        selected items.
         """
         active_handle = self.uistate.get_active(self.navigation_type(),
                                                 self.navigation_group())
@@ -236,7 +237,7 @@ class NavigationView(PageView):
     ####################################################################
     # BOOKMARKS
     ####################################################################
-    def add_bookmark(self, obj):
+    def add_bookmark(self, *obj):
         """
         Add a bookmark to the list.
         """
@@ -247,17 +248,17 @@ class NavigationView(PageView):
         if active_person:
             self.bookmarks.add(active_handle)
             name = name_displayer.display(active_person)
-            self.uistate.push_message(self.dbstate, 
+            self.uistate.push_message(self.dbstate,
                                       _("%s has been bookmarked") % name)
         else:
             from ..dialog import WarningDialog
             WarningDialog(
-                _("Could Not Set a Bookmark"), 
+                _("Could Not Set a Bookmark"),
                 _("A bookmark could not be set because "
                   "no one was selected."),
                 parent=self.uistate.window)
 
-    def edit_bookmarks(self, obj):
+    def edit_bookmarks(self, *obj):
         """
         Call the bookmark editor.
         """
@@ -269,12 +270,8 @@ class NavigationView(PageView):
         """
         self.book_action = ActionGroup(name=self.title + '/Bookmark')
         self.book_action.add_actions([
-            ('AddBook', 'gramps-bookmark-new', _('_Add Bookmark'), 
-             '<PRIMARY>d', None, self.add_bookmark), 
-            ('EditBook', 'gramps-bookmark-edit', 
-             _("%(title)s...") % {'title': _("Organize Bookmarks")}, 
-             '<shift><PRIMARY>D', None, 
-             self.edit_bookmarks), 
+            ('AddBook', self.add_bookmark, '<PRIMARY>d'),
+            ('EditBook', self.edit_bookmarks, '<shift><PRIMARY>D'),
             ])
 
         self._add_action_group(self.book_action)
@@ -288,35 +285,25 @@ class NavigationView(PageView):
         """
         # add the Forward action group to handle the Forward button
         self.fwd_action = ActionGroup(name=self.title + '/Forward')
-        self.fwd_action.add_actions([
-            ('Forward', 'go-next', _("_Forward"),
-             "%sRight" % mod_key(), _("Go to the next object in the history"),
-             self.fwd_clicked)
-            ])
+        self.fwd_action.add_actions([('Forward', self.fwd_clicked,
+                                      "%sRight" % mod_key())])
 
         # add the Backward action group to handle the Forward button
         self.back_action = ActionGroup(name=self.title + '/Backward')
-        self.back_action.add_actions([
-            ('Back', 'go-previous', _("_Back"),
-             "%sLeft" % mod_key(), _("Go to the previous object in the history"),
-             self.back_clicked)
-            ])
+        self.back_action.add_actions([('Back', self.back_clicked,
+                                       "%sLeft" % mod_key())])
 
-        self._add_action('HomePerson', 'go-home', _("_Home"),
-                         accel="%sHome" % mod_key(),
-                         tip=_("Go to the default person"), callback=self.home)
+        self._add_action('HomePerson', self.home, "%sHome" % mod_key())
 
         self.other_action = ActionGroup(name=self.title + '/PersonOther')
         self.other_action.add_actions([
-                ('SetActive', 'go-home', _("Set _Home Person"), None,
-                 None, self.set_default_person),
-                ])
+            ('SetActive', self.set_default_person)])
 
         self._add_action_group(self.back_action)
         self._add_action_group(self.fwd_action)
         self._add_action_group(self.other_action)
 
-    def set_default_person(self, obj):
+    def set_default_person(self, *obj):
         """
         Set the default person.
         """
@@ -324,7 +311,7 @@ class NavigationView(PageView):
         if active:
             self.dbstate.db.set_default_person_handle(active)
 
-    def home(self, obj):
+    def home(self, *obj):
         """
         Move to the default person.
         """
@@ -333,27 +320,27 @@ class NavigationView(PageView):
             self.change_active(defperson.get_handle())
         else:
             from ..dialog import WarningDialog
-            WarningDialog(_("No Home Person"), 
+            WarningDialog(_("No Home Person"),
                 _("You need to set a 'default person' to go to. "
                   "Select the People View, select the person you want as "
                   "'Home Person', then confirm your choice "
-                  "via the menu Edit ->Set Home Person."), 
+                  "via the menu Edit -> Set Home Person."),
                 parent=self.uistate.window)
 
-    def jump(self):
+    def jump(self, *obj):
         """
         A dialog to move to a Gramps ID entered by the user.
         """
-        dialog = Gtk.Dialog(_('Jump to by Gramps ID'), parent=self.uistate.window)
+        dialog = Gtk.Dialog(_('Jump to by Gramps ID'), self.uistate.window)
         dialog.set_border_width(12)
-        label = Gtk.Label(label='<span weight="bold" size="larger">%s</span>' % 
+        label = Gtk.Label(label='<span weight="bold" size="larger">%s</span>' %
                           _('Jump to by Gramps ID'))
         label.set_use_markup(True)
         dialog.vbox.add(label)
         dialog.vbox.set_spacing(10)
         dialog.vbox.set_border_width(12)
         hbox = Gtk.Box()
-        hbox.pack_start(Gtk.Label(label="%s: " % _('ID')), True, True, 0)
+        hbox.pack_start(Gtk.Label(label=_("%s: ") % _('ID')), True, True, 0)
         text = Gtk.Entry()
         text.set_activates_default(True)
         hbox.pack_start(text, False, True, 0)
@@ -362,7 +349,7 @@ class NavigationView(PageView):
                            _('_Jump to'), Gtk.ResponseType.OK)
         dialog.set_default_response(Gtk.ResponseType.OK)
         dialog.vbox.show_all()
-        
+
         if dialog.run() == Gtk.ResponseType.OK:
             gid = text.get_text()
             handle = self.get_handle_from_gramps_id(gid)
@@ -370,7 +357,7 @@ class NavigationView(PageView):
                 self.change_active(handle)
             else:
                 self.uistate.push_message(
-                    self.dbstate, 
+                    self.dbstate,
                     _("Error: %s is not a valid Gramps ID") % gid)
         dialog.destroy()
 
@@ -381,7 +368,7 @@ class NavigationView(PageView):
         """
         pass
 
-    def fwd_clicked(self, obj):
+    def fwd_clicked(self, *obj):
         """
         Move forward one object in the history.
         """
@@ -390,11 +377,12 @@ class NavigationView(PageView):
         if not hobj.at_end():
             hobj.forward()
             self.uistate.modify_statusbar(self.dbstate)
-        self.fwd_action.set_sensitive(not hobj.at_end())
-        self.back_action.set_sensitive(True)
+        self.uimanager.set_actions_sensitive(self.fwd_action,
+                                             not hobj.at_end())
+        self.uimanager.set_actions_sensitive(self.back_action, True)
         hobj.lock = False
 
-    def back_clicked(self, obj):
+    def back_clicked(self, *obj):
         """
         Move backward one object in the history.
         """
@@ -403,114 +391,92 @@ class NavigationView(PageView):
         if not hobj.at_front():
             hobj.back()
             self.uistate.modify_statusbar(self.dbstate)
-        self.back_action.set_sensitive(not hobj.at_front())
-        self.fwd_action.set_sensitive(True)
+        self.uimanager.set_actions_sensitive(self.back_action,
+                                             not hobj.at_front())
+        self.uimanager.set_actions_sensitive(self.fwd_action, True)
         hobj.lock = False
-        
+
     ####################################################################
     # MRU functions
     ####################################################################
-    
+
     def mru_disable(self):
         """
         Remove the UI and action groups for the MRU list.
         """
         if self.mru_active != DISABLED:
-            self.uistate.uimanager.remove_ui(self.mru_active)
-            self.uistate.uimanager.remove_action_group(self.mru_action)
+            self.uimanager.remove_ui(self.mru_active)
+            self.uimanager.remove_action_group(self.mru_action)
             self.mru_active = DISABLED
 
-    def mru_enable(self):
+    def mru_enable(self, update_menu=False):
         """
         Enables the UI and action groups for the MRU list.
         """
         if self.mru_active == DISABLED:
-            self.uistate.uimanager.insert_action_group(self.mru_action, 1)
-            self.mru_active = self.uistate.uimanager.add_ui_from_string(self.mru_ui)
-            self.uistate.uimanager.ensure_update()
+            self.uimanager.insert_action_group(self.mru_action)
+            self.mru_active = self.uimanager.add_ui_from_string(self.mru_ui)
+            if update_menu:
+                self.uimanager.update_menu()
 
-    def update_mru_menu(self, items):
+    def update_mru_menu(self, items, update_menu=True):
         """
         Builds the UI and action group for the MRU list.
         """
+        menuitem = '''        <item>
+              <attribute name="action">win.%s%02d</attribute>
+              <attribute name="label">%s</attribute>
+            </item>
+            '''
+        menus = ''
         self.mru_disable()
         nav_type = self.navigation_type()
         hobj = self.get_history()
         menu_len = min(len(items) - 1, MRU_SIZE)
-        
-        entry = '<menuitem action="%s%02d"/>'
-        data = [entry % (nav_type, index) for index in range(0, menu_len)]
-        self.mru_ui = "".join(MRU_TOP) + "".join(data) + "".join(MRU_BTM)
-        
+
+        for index in range(0, menu_len):
+            name, obj = navigation_label(self.dbstate.db, nav_type,
+                                         items[index])
+            menus += menuitem % (nav_type, index, html.escape(name))
+        self.mru_ui = [MRU_TOP + menus + MRU_BTM]
+
         mitems = items[-MRU_SIZE - 1:-1] # Ignore current handle
         mitems.reverse()
         data = []
         for index, handle in enumerate(mitems):
-            name, obj = navigation_label(self.dbstate.db, nav_type, handle)
-            data.append(('%s%02d'%(nav_type, index), None,  name,
-                         "%s%d" % (mod_key(), index), None,
-                         make_callback(hobj.push, handle)))
- 
+            data.append(('%s%02d'%(nav_type, index),
+                         make_callback(hobj.push, handle),
+                         "%s%d" % (mod_key(), index)))
+
         self.mru_action = ActionGroup(name=self.title + '/MRU')
         self.mru_action.add_actions(data)
-        self.mru_enable()
+        self.mru_enable(update_menu)
 
     ####################################################################
     # Template functions
     ####################################################################
-    def get_bookmarks(self):
-        """
-        Template function to get bookmarks.
-        We could implement this here based on navigation_type()
-        """
-        raise NotImplementedError
-        
-    def edit(self, obj):
-        """
-        Template function to allow the editing of the selected object
-        """
-        raise NotImplementedError
-
-    def remove(self, handle):
-        """
-        Template function to allow the removal of an object by its handle
-        """
-        raise NotImplementedError
-
-    def add(self, obj):
-        """
-        Template function to allow the adding of a new object
-        """
-        raise NotImplementedError
-
-    def remove_object_from_handle(self, handle):
-        """
-        Template function to allow the removal of an object by its handle
-        """
-        raise NotImplementedError
-
+    @abstractmethod
     def build_tree(self):
         """
         Rebuilds the current display. This must be overridden by the derived
         class.
         """
-        raise NotImplementedError
 
+    @abstractmethod
     def build_widget(self):
         """
         Builds the container widget for the interface. Must be overridden by the
         the base class. Returns a gtk container widget.
         """
-        raise NotImplementedError
-        
+
     def key_press_handler(self, widget, event):
         """
         Handle the control+c (copy) and control+v (paste), or pass it on.
         """
         if self.active:
             if event.type == Gdk.EventType.KEY_PRESS:
-                if (event.keyval == Gdk.KEY_c and 
-                    (event.get_state() & Gdk.ModifierType.CONTROL_MASK)):
+                if (event.keyval == Gdk.KEY_c and
+                    match_primary_mask(event.get_state())):
                     self.call_copy()
                     return True
         return super(NavigationView, self).key_press_handler(widget, event)
@@ -532,4 +498,4 @@ def make_callback(func, handle):
     """
     Generates a callback function based off the passed arguments
     """
-    return lambda x: func(handle)
+    return lambda x, y: func(handle)
